

package cs235.columnsort;

import java.util.Arrays;
import java.util.Random;


public class TestDriver {

    static final int CountIndex = 0;
    static final int ColumnsIndex = 1;
    static final int IterationsIndex = 2;
    static final int SeedIndex = 3;

    static final int DefaultCount = 1000000;

    static final int DescCount = 27;
    static final int DescColumns = 3;
    static final int DescIterations = 1;
    static final int DescRows = 9;
    static final int DescPaddedCount = 27;
    static final short[] DescData = new short[] {2, 3, 8, 10, 14, 19, 21, 24, 26,
				      1, 4, 12, 13, 15, 17, 20, 23, 25,
				      5, 6, 7, 9, 11, 16, 18, 22, 27};

    static final String outOfMemoryMessage = 
	"Consider running the java interpreter with the -Xmx option\n" +
	"to increase the amount of allocated memory.\n" +
	"(See project description)";

/**
	Run Column Sort with randomly-generated test data.
	<p>
	USAGE: java cs235.columnsort.TestDriver [count [columns [iterations [seed]]]]
	<p>
	If no command-line arguments are specified, the test driver runs the example 
	from the Column Sort algorithm description.

	@param args[0] The number of elements to be sorted.  This number may be adjusted
				upward in order to satisfy Column Sort's constraints on the number
				of rows and columns.
	@param args[1] The number of columns to be included in the generated data.  The
				number of rows is inferred from the number of elements and the
				number of columns (rows = count / columns).
	@param args[2] The number of times that Column Sort should be used to sort the
				data.  The reported running time is the minimum over all runs.
	@param args[3] The seed to be used with the random number generator that is used
				to generate test data.
*/
    public static void main(String[] args) {
	try {
	    int count = DefaultCount;
	    int columns = 1;
	    int iterations = 1;
	    long seed = System.currentTimeMillis();
	    int rows = 0;
	    int paddedCount = 0;
	    short[] data = null;

	    if (args.length > CountIndex) {
		count = Integer.parseInt(args[CountIndex]);
		if (args.length > ColumnsIndex)
		    columns = Integer.parseInt(args[ColumnsIndex]);
		if (args.length > IterationsIndex)
		    iterations = Math.max(Integer.parseInt(args[IterationsIndex]), 1);
		if (args.length > SeedIndex)
		    seed = Long.parseLong(args[SeedIndex]);

		rows = getRows(count, columns);

		paddedCount = rows * columns;

		data = generateData(paddedCount, seed);
	    } else {
		// setup the example from the algorithm description

		count = DescCount;
		columns = DescColumns;
		iterations = DescIterations;
		rows = DescRows;
		paddedCount = DescPaddedCount;
		data = DescData;
	    }

	    System.out.println("padded count: " + paddedCount);
	    System.out.println("rows: " + rows);
	    System.out.println("cols: " + columns);


	    short[] copy = runColumnSort(data, rows, columns, iterations);

	    runBuiltinSort(data);


	    // verify results of column sort
	    //

	    if (!Arrays.equals(data, copy))
		System.out.println("column sort did not sort the data correctly");
	}
	catch (OutOfMemoryError me)	{
	    me.printStackTrace();
	    System.err.println(outOfMemoryMessage);
	    System.exit(-1);
	}
    }

    private static int getRows(int count, int columns) {
	int rows;

	// ensure that rows * columns >= count
	rows = (int)Math.ceil((double)count / (double)columns);

	// ensure that rows >= 2*(columns-1)^2
	rows = Math.max(rows, 2 * (columns-1) * (columns-1));

	// ensure that rows % columns == 0
	if ((rows % columns) != 0)
	    rows = (rows / columns + 1) * columns;

	return rows;
    }

    private static short[] generateData(int count, long seed) {
	Random rand = new Random(seed);
	short[] data = new short[count];
	for (int i = 0; i < count; i++) {
	    data[i] = (short)rand.nextInt(Short.MAX_VALUE);
	}
	return data;
    }

    private static short[] runColumnSort(short[] data, int rows, int columns, int iterations) {
	ColumnSort sorter = ColumnSortFactory.create();

	short[] copy = null;
	long start = 0;
	long duration = 0;

	long minDuration = Integer.MAX_VALUE;
	for (int i=0; i < iterations; ++i) {
	    copy = (short[])data.clone();
	    start = System.currentTimeMillis();
	    sorter.run(copy, rows, columns);
	    duration = System.currentTimeMillis() - start;
	    minDuration = Math.min(duration, minDuration);
	}

	System.out.println("column sort:   " + minDuration + " ms");
	return copy;
    }

    private static void runBuiltinSort(short[] data) {
	long start = System.currentTimeMillis();
	Arrays.sort(data);
	long duration = System.currentTimeMillis() - start;

	System.out.println("built-in sort: " + duration + " ms");
    }

}

